/*      
        alph: Piper program implementing and analysing traditional and
        historic c(ai)phers designed to be used in conjuction with other
        programs.

        Copyright (C) 2004  Corcalciuc V. Horia

        This library is free software; you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public
        License as published by the Free Software Foundation; either
        version 2.1 of the License, or (at your option) any later version.

        This library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
        Lesser General Public License for more details.

        You should have received a copy of the GNU Lesser General Public
        License along with this library; if not, write to the Free Software
        Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

        Dedicated to Rudolf.
*/


/* System */
#include <stdio.h>
#include <string.h>

/* Our own. */
#include "alphCRYPT.h"
#include "alphan.h"
#include "config.h"

int main(argc, argv)
int argc;
char **argv;
{

  int optr;
  int forw;
  char *key[5];

  char *copts[] = { "--CAESAR",
                    "--ATBASH", 
                    "--ROT13", 
                    "--VIGENERE",
                    "--VERNAM",
                    "--PLAYFAIR",
                    "--BIFID",
                    "--ENIGMA"
                  };

  char cid[] = { 'c', 'a', 'r', 'v', 'u', 'p', 'b', 'e' };
  char *copts_desc[] = { "",
                         "",
                         "",
                         "ALPHA KEY",
                         "FILE",
                         "ALPHA KEY",
                         "ALPHA KEY",
                         "ROTORS,REFLECTOR,RING"
                       };
  char tgg[] = { 'e', 'd', 's' };
  char *tgg_desc[] = { "encrypt the stream",
                       "decrypt the stream",
                       "try to smash stream"
                     };
  typedef enum { false = 0, true } flip;
  flip dsmash = false;
  char *indata;
  int incr;
  FILE *fp;

  for (optr=argc-1; optr >= 0; optr--)
    if (strlen (argv[optr]) < 7)
      if (strchr(argv[optr], 'h') || strstr(argv[optr], "help"))
        {
            printf ("%s v%s:\n", PACKAGE_NAME, PACKAGE_VERSION);
            printf ("\nCiphers:\n");
            for (forw=0; forw < 8; forw++)
              printf ("\t%s %s\n", copts[forw], copts_desc[forw]);
            printf("\nOptions:\n");
            for (forw=0; forw < 3; forw++)
              printf ("\t-%c %s\n", tgg[forw], tgg_desc[forw]);
            printf ("\n(c) Corcalciuc V. Horia - rain@sevenrains.ro\n");
            exit(0);
        }

  indata = calloc (BUFSIZ, sizeof(char));

  fflush(stdin);
  for (optr = 0; optr <= BUFSIZ; optr++)
    if (!feof (stdin))
      indata[optr] = fgetc (stdin);
    else
      {
        indata[optr -1] = '\0';
        break;
      }

  while (argc-->0)
    {
      for (optr=0; optr<8; optr++)
        {
          if (!strcmp(copts[optr],argv[argc]-'\0'))
            switch (cid[optr])
              {
                case 'c':
                  caesar (indata);
                  break;
                case 'a':
                  atbash (indata);
                  break;
                case 'r':
                  rot13 (indata);
                  break;
                case 'v':
                  key[0] = (char *) calloc (strlen(argv[argc+1]), sizeof(char));
                  strcpy (key[0], argv[argc+1]);
                  vigenere (indata, key[0]);
                  break;
                case 'u':
                  incr = 0;
                  key[0] = (char *) calloc (strlen(indata), sizeof(char));
                  if ((fp = fopen (argv[argc+1], "r")) == NULL)
                    exit(1);
                  else
                    while ((forw = fgetc(fp)) != EOF && incr < strlen(indata))
                      if (isalpha (forw))
                        key[0][incr++] = forw;
                  vernam (indata, key[0]);
                  break;
                case 'p':
                  key[0] = (char *) calloc (strlen(argv[argc+1]), sizeof(char));
                  for (incr=0; incr <= strlen(argv[argc+1]); incr++)
                    key[0][incr]=argv[argc+1][incr];
                  playfair (indata, key[0]);
                  break;
                case 'b':
                  key[0] = (char *) calloc (strlen(argv[argc+1]), sizeof(char));
                  for (incr=0; incr <= strlen(argv[argc+1]); incr++)
                    key[0][incr]=argv[argc+1][incr];
                  bifid (indata, key[0]);
                  break;
                case 'e':
                  for (forw=1; forw<4; forw++)
                  {
                    if (forw==1 || forw == 3)
                      if (strlen(argv[argc+forw])<3 || strlen(argv[argc+forw])>3)
                        printf("%s: %s\n", PACKAGE_NAME, KEYB);
                    key[forw-1] = (char *) calloc (strlen(argv[argc+forw]), sizeof(char));
                    strcpy ( key[forw-1], argv[argc+forw] );
                  }
                  enigma ( indata, key[0], key[1], key[2] );
                  break;
              }
          else if (strlen(argv[argc])<3 && argv[argc][1]==tgg[optr])
            {
              switch (tgg[optr])
                {
                  case 'e':
                    al_dec=false;
                    break;
                  case 'd':
                    al_dec=true;
                    break;
                  case 's':
                    dsmash=true;
                    break;
                }
            }
        }
    }

  if (dsmash)
    {
      printf("%s", smash(indata));
    }
  else
    printf("%s", indata);

  exit(0);

}

